@@ -21,19 +21,8 @@ import (
 	"fmt"
 	"reflect"
 
-	configv1 "github.com/openshift/api/config/v1"
-	operatorv1 "github.com/openshift/api/operator/v1"
 	log "github.com/sirupsen/logrus"
 
-	"sigs.k8s.io/controller-runtime/pkg/client"
-
-	minterv1 "github.com/openshift/cloud-credential-operator/pkg/apis/cloudcredential/v1"
-	ccgcp "github.com/openshift/cloud-credential-operator/pkg/gcp"
-	"github.com/openshift/cloud-credential-operator/pkg/operator/constants"
-	actuatoriface "github.com/openshift/cloud-credential-operator/pkg/operator/credentialsrequest/actuator"
-	"github.com/openshift/cloud-credential-operator/pkg/operator/utils"
-	gcputils "github.com/openshift/cloud-credential-operator/pkg/operator/utils/gcp"
-
 	// GCP packages
 	iamadminpb "google.golang.org/genproto/googleapis/iam/admin/v1"
 	"google.golang.org/grpc/codes"
@@ -44,11 +33,23 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
+
+	"sigs.k8s.io/controller-runtime/pkg/client"
+
+	configv1 "github.com/openshift/api/config/v1"
+	operatorv1 "github.com/openshift/api/operator/v1"
+
+	minterv1 "github.com/openshift/cloud-credential-operator/pkg/apis/cloudcredential/v1"
+	ccgcp "github.com/openshift/cloud-credential-operator/pkg/gcp"
+	"github.com/openshift/cloud-credential-operator/pkg/operator/constants"
+	actuatoriface "github.com/openshift/cloud-credential-operator/pkg/operator/credentialsrequest/actuator"
+	"github.com/openshift/cloud-credential-operator/pkg/operator/utils"
+	gcputils "github.com/openshift/cloud-credential-operator/pkg/operator/utils/gcp"
 )
 
 const (
 	roGCPCredsSecretNamespace = "openshift-cloud-credential-operator"
-	roGCPCredsSecret          = "cloud-credential-operator-iam-ro-creds"
+	roGCPCredsSecret          = "cloud-credential-operator-gcp-ro-creds"
 
 	gcpSecretJSONKey = "service_account.json"
 )
@@ -177,7 +178,7 @@ func (a *Actuator) sync(ctx context.Context, cr *minterv1.CredentialsRequest) er
 		logger.WithError(err).Error("error determining whether a credentials update is needed")
 		return &actuatoriface.ActuatorError{
 			ErrReason: minterv1.CredentialsProvisionFailure,
-			Message:   fmt.Sprintf("error determining whether a credentials update is needed: %v", err),
+			Message:   "error determining whether a credentials update is needed",
 		}
 	}
 
@@ -195,13 +196,13 @@ func (a *Actuator) sync(ctx context.Context, cr *minterv1.CredentialsRequest) er
 		return nil
 	}
 
-	cloudCredsSecret, err := a.getRootCloudCredentialsSecret(ctx, logger)
+	credentialsRootSecret, err := a.GetCredentialsRootSecret(ctx, cr)
 	if err != nil {
 		logger.WithError(err).Error("issue with cloud credentials secret")
 		return err
 	}
 
-	switch cloudCredsSecret.Annotations[constants.AnnotationKey] {
+	switch credentialsRootSecret.Annotations[constants.AnnotationKey] {
 	case constants.InsufficientAnnotation:
 		msg := "cloud credentials insufficient to satisfy credentials request"
 		logger.Error(msg)
@@ -211,7 +212,7 @@ func (a *Actuator) sync(ctx context.Context, cr *minterv1.CredentialsRequest) er
 		}
 	case constants.PassthroughAnnotation:
 		logger.Debug("provisioning with passthrough")
-		err := a.syncPassthrough(ctx, cr, cloudCredsSecret, logger)
+		err := a.syncPassthrough(ctx, cr, credentialsRootSecret, logger)
 		if err != nil {
 			return err
 		}
@@ -320,16 +321,23 @@ func (a *Actuator) syncMint(ctx context.Context, cr *minterv1.CredentialsRequest
 		return fmt.Errorf("error gathering permissions for each role: %v", err)
 	}
 
-	// check that service APIs are enabled before we bother to make a
-	// new serviceaccount
-	serviceAPIsEnabled, err := checkServicesEnabled(rootGCPClient, permList, logger)
-	if err != nil {
-		return err
+	var serviceAPIsEnabled bool
+	if gcpSpec.SkipServiceCheck {
+		// Since we are skipping the checks, we will assume that the APIS are enabled.
+		serviceAPIsEnabled = true
+	} else {
+		// check that service APIs are enabled before we bother to make a
+		// new serviceaccount
+		var err error
+		serviceAPIsEnabled, err = checkServicesEnabled(rootGCPClient, permList, logger)
+		if err != nil {
+			return err
+		}
 	}
-	if !gcpSpec.SkipServiceCheck && !serviceAPIsEnabled {
+
+	if !serviceAPIsEnabled {
 		return fmt.Errorf("not all required service APIs are enabled")
 	}
-	serviceAPIsEnabled = true
 
 	// Create service account if necessary
 	var serviceAccount *iamadminpb.ServiceAccount
@@ -389,7 +397,7 @@ func (a *Actuator) syncMint(ctx context.Context, cr *minterv1.CredentialsRequest
 }
 
 // needsUpdate will return a bool indicated that all the applicable service APIs are enabled,
-// a bool indicat whether any update to existing perms are needed, and any error encountered.
+// a bool indicating whether any update to existing perms are needed, and any error encountered.
 func (a *Actuator) needsUpdate(ctx context.Context, cr *minterv1.CredentialsRequest) (bool, bool, error) {
 	logger := a.getLogger(cr)
 
@@ -403,8 +411,7 @@ func (a *Actuator) needsUpdate(ctx context.Context, cr *minterv1.CredentialsRequ
 		return true, false, fmt.Errorf("unable to decode ProviderStatus: %v", err)
 	}
 
-	// TODO: change to a real read-only client once we have ro-creds solution
-	readClient, err := a.buildRootGCPClient(cr)
+	readClient, err := a.buildReadGCPClient(cr)
 	if err != nil {
 		log.WithError(err).Error("error creating GCP client")
 		return false, true, fmt.Errorf("unable to check whether credentialsRequest needs update")
@@ -416,16 +423,22 @@ func (a *Actuator) needsUpdate(ctx context.Context, cr *minterv1.CredentialsRequ
 		return false, true, fmt.Errorf("error gathering permissions for each role: %v", err)
 	}
 
+	var serviceAPIsEnabled bool
 	// Are the service APIs enabled
-	serviceAPIsEnabled, err := checkServicesEnabled(readClient, permList, logger)
-	if err != nil {
-		return false, true, fmt.Errorf("error checking whether service APIs are enabled: %v", err)
+	if gcpSpec.SkipServiceCheck {
+		// Since we are skipping the checks, we will assume that the APIs are enabled.
+		serviceAPIsEnabled = true
+	} else {
+		var err error
+		serviceAPIsEnabled, err = checkServicesEnabled(readClient, permList, logger)
+		if err != nil {
+			return false, true, fmt.Errorf("error checking whether service APIs are enabled: %v", err)
+		}
 	}
 
-	if !gcpSpec.SkipServiceCheck && !serviceAPIsEnabled {
+	if !serviceAPIsEnabled {
 		return serviceAPIsEnabled, true, nil
 	}
-	serviceAPIsEnabled = true
 
 	// If the secret simply doesn't exist, we definitely need an update
 	exists, err := a.Exists(ctx, cr)
@@ -469,6 +482,17 @@ func (a *Actuator) needsUpdate(ctx context.Context, cr *minterv1.CredentialsRequ
 	} else {
 		// passthrough-specifc check here
 
+		credentialsRootSecret, err := a.GetCredentialsRootSecret(ctx, cr)
+		if err != nil {
+			log.WithError(err).Debug("error retrieving cloud credentials secret")
+			return serviceAPIsEnabled, false, err
+		}
+		// If the cloud credentials secret has been updated in passthrough mode, we need an update
+		if credentialsRootSecret != nil && credentialsRootSecret.ResourceVersion != cr.Status.LastSyncCloudCredsSecretResourceVersion {
+			logger.Debug("root cloud creds have changed, update is needed")
+			return serviceAPIsEnabled, true, nil
+		}
+
 		allowed, err := gcputils.CheckPermissionsAgainstPermissionList(readClient, permList, logger)
 		if err != nil {
 			return serviceAPIsEnabled, true, fmt.Errorf("error checking whether GCP client has sufficient permissions: %v", err)
@@ -493,6 +517,31 @@ func (a *Actuator) needsUpdate(ctx context.Context, cr *minterv1.CredentialsRequ
 	return serviceAPIsEnabled, false, nil
 }
 
+// buildReadGCPClient will return a GCP client using the scaled down read only GCP creds
+// for CCO, which are expected to live in openshift-cloud-credential-perator/cloud-credential-operator-gcp-ro-creds.
+// These creds would normally be created by CCO itself, via a CredentialsRequest created
+// while installing CCO.
+//
+// If these are not available but root creds are, we will use the root creds instead.
+// This allows us to create the read creds initially.
+func (a *Actuator) buildReadGCPClient(cr *minterv1.CredentialsRequest) (ccgcp.Client, error) {
+	logger := a.getLogger(cr).WithField("secret", fmt.Sprintf("%s/%s", roGCPCredsSecretNamespace, roGCPCredsSecret))
+	logger.Debug("loading GCP read-only credentials from secret")
+
+	jsonBytes, err := loadCredsFromSecret(a.Client, roGCPCredsSecretNamespace, roGCPCredsSecret)
+	if err != nil {
+		if errors.IsNotFound(err) {
+			logger.Warn("read-only creds not found, using root creds client")
+			return a.buildRootGCPClient(cr)
+		}
+		logger.WithError(err).Error("failed to load in read-only creds Secret")
+		return nil, err
+	}
+
+	logger.Debug("creating read GCP client")
+	return a.GCPClientBuilder(a.ProjectName, jsonBytes)
+}
+
 func (a *Actuator) buildRootGCPClient(cr *minterv1.CredentialsRequest) (ccgcp.Client, error) {
 	logger := a.getLogger(cr).WithField("secret", fmt.Sprintf("%s/%s", constants.CloudCredSecretNamespace, constants.GCPCloudCredSecretName))
 
@@ -506,41 +555,6 @@ func (a *Actuator) buildRootGCPClient(cr *minterv1.CredentialsRequest) (ccgcp.Cl
 	return a.GCPClientBuilder(a.ProjectName, jsonBytes)
 }
 
-func (a *Actuator) buildReadGCPClient(cr *minterv1.CredentialsRequest) (ccgcp.Client, error) {
-	logger := a.getLogger(cr).WithField("secret", fmt.Sprintf("%s/%s", roGCPCredsSecretNamespace, roGCPCredsSecret))
-	logger.Debug("loading GCP credentials from secret")
-
-	var err error
-	var jsonBytes []byte
-
-	if cr.Spec.SecretRef.Name == roGCPCredsSecret && cr.Spec.SecretRef.Namespace == roGCPCredsSecretNamespace {
-		log.Debug("operating our own RO creds, using root creds for GCP client operator")
-		jsonBytes, err = loadCredsFromSecret(a.Client, constants.CloudCredSecretNamespace, constants.GCPCloudCredSecretName)
-		if err != nil {
-			return nil, err
-		}
-	} else {
-		jsonBytes, err = loadCredsFromSecret(a.Client, roGCPCredsSecretNamespace, roGCPCredsSecret)
-		if err != nil {
-			if errors.IsNotFound(err) {
-				logger.Warn("read-only creds not found, using root creds client")
-				return a.buildRootGCPClient(cr)
-			} else {
-				logger.Errorf("error reading in read-only creds: %v", err)
-				return nil, err
-			}
-		}
-	}
-
-	logger.Debug("creating read GCP client")
-	client, err := a.GCPClientBuilder(a.ProjectName, jsonBytes)
-	if err != nil {
-		return nil, err
-	}
-
-	return client, nil
-}
-
 func (a *Actuator) updateProviderStatus(ctx context.Context, logger log.FieldLogger, cr *minterv1.CredentialsRequest, gcpStatus *minterv1.GCPProviderStatus) error {
 	var err error
 	cr.Status.ProviderStatus, err = a.Codec.EncodeProviderStatus(gcpStatus)
@@ -566,8 +580,9 @@ func (a *Actuator) GetCredentialsRootSecretLocation() types.NamespacedName {
 	return types.NamespacedName{Namespace: constants.CloudCredSecretNamespace, Name: constants.GCPCloudCredSecretName}
 }
 
-// getRootCloudCredentialsSecret will return the cluster's root GCP cloud cred secret if it exists and is properly annotated
-func (a *Actuator) getRootCloudCredentialsSecret(ctx context.Context, logger log.FieldLogger) (*corev1.Secret, error) {
+// GetCredentialsRootSecret will return the cluster's root GCP cloud cred secret if it exists and is properly annotated
+func (a *Actuator) GetCredentialsRootSecret(ctx context.Context, cr *minterv1.CredentialsRequest) (*corev1.Secret, error) {
+	logger := a.getLogger(cr)
 	cloudCredSecret := &corev1.Secret{}
 	if err := a.Client.Get(ctx, a.GetCredentialsRootSecretLocation(), cloudCredSecret); err != nil {
 		msg := "unable to fetch root cloud cred secret"
@@ -639,7 +654,7 @@ func (a *Actuator) secretAlreadySynced(cr *minterv1.CredentialsRequest) (bool, e
 		return false, err
 	}
 
-	rootSecret, err := a.getRootCloudCredentialsSecret(context.TODO(), logger)
+	rootSecret, err := a.GetCredentialsRootSecret(context.TODO(), cr)
 	if err != nil {
 		logger.WithError(err).Error("error retrieving cluster cloud creds")
 		return false, err
@@ -770,14 +785,13 @@ func checkServicesEnabled(gcpClient ccgcp.Client, permList []string, logger log.
 	return serviceAPIsEnabled, nil
 }
 
+// Upgradeable returns a ClusterOperator status condition for the upgradeable type
+// if the system is considered not upgradeable. Otherwise, return nil as the default
+// value is for things to be upgradeable.
 func (a *Actuator) Upgradeable(mode operatorv1.CloudCredentialsMode) *configv1.ClusterOperatorStatusCondition {
-	upgradeableCondition := &configv1.ClusterOperatorStatusCondition{
-		Status: configv1.ConditionTrue,
-		Type:   configv1.OperatorUpgradeable,
-	}
-	return upgradeableCondition
+	return utils.UpgradeableCheck(a.Client, mode, "4.7", a.GetUpcomingCredSecrets(), a.GetCredentialsRootSecretLocation())
 }
 
 func (a *Actuator) GetUpcomingCredSecrets() []types.NamespacedName {
-	return []types.NamespacedName{}
+	return constants.GCPUpcomingSecrets
 }
