@@ -25,17 +25,6 @@ import (
 	log "github.com/sirupsen/logrus"
 	"golang.org/x/time/rate"
 
-	minterv1 "github.com/openshift/cloud-credential-operator/pkg/apis/cloudcredential/v1"
-	"github.com/openshift/cloud-credential-operator/pkg/operator/constants"
-	"github.com/openshift/cloud-credential-operator/pkg/operator/credentialsrequest/actuator"
-	"github.com/openshift/cloud-credential-operator/pkg/operator/internalcontroller"
-	"github.com/openshift/cloud-credential-operator/pkg/operator/metrics"
-	"github.com/openshift/cloud-credential-operator/pkg/operator/utils"
-	"github.com/openshift/cloud-credential-operator/pkg/util/clusteroperator"
-
-	configv1 "github.com/openshift/api/config/v1"
-	operatorv1 "github.com/openshift/api/operator/v1"
-
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -51,6 +40,17 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/predicate"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 	"sigs.k8s.io/controller-runtime/pkg/source"
+
+	configv1 "github.com/openshift/api/config/v1"
+	operatorv1 "github.com/openshift/api/operator/v1"
+
+	minterv1 "github.com/openshift/cloud-credential-operator/pkg/apis/cloudcredential/v1"
+	"github.com/openshift/cloud-credential-operator/pkg/operator/constants"
+	"github.com/openshift/cloud-credential-operator/pkg/operator/credentialsrequest/actuator"
+	"github.com/openshift/cloud-credential-operator/pkg/operator/internalcontroller"
+	"github.com/openshift/cloud-credential-operator/pkg/operator/metrics"
+	"github.com/openshift/cloud-credential-operator/pkg/operator/status"
+	"github.com/openshift/cloud-credential-operator/pkg/operator/utils"
 )
 
 const (
@@ -69,11 +69,6 @@ const (
 	cloudCredDeprovisionSuccess = "CloudCredDeprovisionSuccess"
 
 	credentialsRequestInfraMismatch = "InfrastructureMismatch"
-
-	// Hack: send a fake CredentialsRequest to get us a status update when we don't have one to reconcile.
-	// Should be removed when status update is broken out into it's own controller.
-	fakeCredRequestName     = "no-op-fake-cred-request"
-	fakeCredReuestNamespace = "no-op-fake-cred-request-namespace"
 )
 
 var (
@@ -97,18 +92,9 @@ func newReconciler(mgr manager.Manager, actuator actuator.Actuator, platType con
 		Actuator:     actuator,
 		platformType: platType,
 	}
-	clusteroperator.AddStatusHandler(r)
-	return r
-}
+	status.AddHandler(controllerName, r)
 
-// isFutureCredRequestSecret is used to identify if a given secret namespace + name is one we're expecting in a future
-func isFutureCredRequestSecret(a actuator.Actuator, namespace, name string) bool {
-	for _, nsn := range a.GetUpcomingCredSecrets() {
-		if nsn.Namespace == namespace && nsn.Name == name {
-			return true
-		}
-	}
-	return false
+	return r
 }
 
 // add adds a new Controller to mgr with r as the reconcile.Reconciler
@@ -151,8 +137,6 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 		return err
 	}
 
-	actuator := (r.(*ReconcileCredentialsRequest)).Actuator
-
 	// Define a mapping for secrets to the credentials requests that created them. (if applicable)
 	// We use an annotation on secrets that refers back to their owning credentials request because
 	// the normal owner reference is not namespaced, and we want to support credentials requests being
@@ -160,17 +144,6 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 	targetCredSecretMapFunc := handler.ToRequestsFunc(
 		func(a handler.MapObject) []reconcile.Request {
 
-			// Hack: reconcile a fake cred request we're watching for in the Reconcile loop, intended to be
-			// a no-op so we can get a status update. (remove once we break status out into it's own controller)
-			if isFutureCredRequestSecret(actuator, a.Meta.GetNamespace(), a.Meta.GetName()) {
-				return []reconcile.Request{
-					{NamespacedName: types.NamespacedName{
-						Name:      fakeCredRequestName,
-						Namespace: fakeCredReuestNamespace,
-					}},
-				}
-			}
-
 			// Predicate below should ensure this map function is not called if the
 			// secret does not have our label:
 			namespace, name, err := cache.SplitMetaNamespaceKey(a.Meta.GetAnnotations()[minterv1.AnnotationCredentialsRequest])
@@ -194,11 +167,6 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 	p := predicate.Funcs{
 		UpdateFunc: func(e event.UpdateEvent) bool {
 
-			// Watch for Secrets from future release cred requests:
-			if isFutureCredRequestSecret(actuator, e.MetaNew.GetNamespace(), e.MetaNew.GetName()) {
-				return true
-			}
-
 			// The object doesn't contain our label, so we have nothing to reconcile.
 			if _, ok := e.MetaOld.GetAnnotations()[minterv1.AnnotationCredentialsRequest]; !ok {
 				return false
@@ -206,22 +174,12 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 			return true
 		},
 		CreateFunc: func(e event.CreateEvent) bool {
-			// Watch for Secrets from future release cred requests:
-			if isFutureCredRequestSecret(actuator, e.Meta.GetNamespace(), e.Meta.GetName()) {
-				return true
-			}
-
 			if _, ok := e.Meta.GetAnnotations()[minterv1.AnnotationCredentialsRequest]; !ok {
 				return false
 			}
 			return true
 		},
 		DeleteFunc: func(e event.DeleteEvent) bool {
-			// Watch for Secrets from future release cred requests:
-			if isFutureCredRequestSecret(actuator, e.Meta.GetNamespace(), e.Meta.GetName()) {
-				return true
-			}
-
 			if _, ok := e.Meta.GetAnnotations()[minterv1.AnnotationCredentialsRequest]; !ok {
 				return false
 			}
@@ -386,6 +344,7 @@ func isAdminCredSecret(namespace, secretName string) bool {
 			secretName == constants.GCPCloudCredSecretName ||
 			secretName == constants.OpenStackCloudCredsSecretName ||
 			secretName == constants.OvirtCloudCredsSecretName ||
+			secretName == constants.KubevirtCloudCredSecretName ||
 			secretName == constants.VSphereCloudCredSecretName {
 			log.WithField("secret", secretName).WithField("namespace", namespace).Info("observed admin cloud credential secret event")
 			return true
@@ -421,24 +380,11 @@ func (r *ReconcileCredentialsRequest) Reconcile(request reconcile.Request) (reco
 		"cr":         fmt.Sprintf("%s/%s", request.NamespacedName.Namespace, request.NamespacedName.Name),
 	})
 
-	// Ensure we always sync operator status after reconciling, even if an error occurred.
-	// TODO: Move to another controller:
 	defer func() {
-		err := r.syncOperatorStatus()
-		if err != nil {
-			logger.WithError(err).Error("failed to sync operator status")
-		}
 		dur := time.Since(start)
 		metrics.MetricControllerReconcileTime.WithLabelValues(controllerName).Observe(dur.Seconds())
 	}()
 
-	// Hack: reconcile a fake cred request we're watching for in the Reconcile loop, intended to be
-	// a no-op so we can get a status update. (remove once we break status out into it's own controller)
-	if request.NamespacedName.Namespace == fakeCredReuestNamespace && request.NamespacedName.Name == fakeCredRequestName {
-		logger.Info("detected reconcile of fake credentials request name for status update purposes, returning")
-		return reconcile.Result{}, nil
-	}
-
 	mode, conflict, err := utils.GetOperatorConfiguration(r.Client, logger)
 	if err != nil {
 		logger.WithError(err).Error("error checking if operator is disabled")
@@ -593,11 +539,16 @@ func (r *ReconcileCredentialsRequest) Reconcile(request reconcile.Request) (reco
 		crSecretExists = true
 	}
 
+	credentialsRootSecret, err := r.Actuator.GetCredentialsRootSecret(context.TODO(), cr)
+	if err != nil {
+		log.WithError(err).Debug("error retrieving cloud credentials secret, admin can remove root credentials in mint mode")
+	}
+	cloudCredsSecretUpdated := credentialsRootSecret != nil && credentialsRootSecret.ResourceVersion != cr.Status.LastSyncCloudCredsSecretResourceVersion
 	isStale := cr.Generation != cr.Status.LastSyncGeneration
 	hasRecentlySynced := cr.Status.LastSyncTimestamp != nil && cr.Status.LastSyncTimestamp.Add(syncPeriod).After(time.Now())
 	hasActiveFailureConditions := checkForFailureConditions(cr)
 
-	if !isStale && hasRecentlySynced && crSecretExists && !hasActiveFailureConditions && cr.Status.Provisioned {
+	if !cloudCredsSecretUpdated && !isStale && hasRecentlySynced && crSecretExists && !hasActiveFailureConditions && cr.Status.Provisioned {
 		logger.Debug("lastsyncgeneration is current and lastsynctimestamp was less than an hour ago, so no need to sync")
 		// Since we get no events for changes made directly to the cloud/platform, set the requeueAfter so that we at
 		// least periodically check that nothing out in the cloud/platform was modified that would require us to fix up
@@ -641,6 +592,9 @@ func (r *ReconcileCredentialsRequest) Reconcile(request reconcile.Request) (reco
 			Time: time.Now(),
 		}
 		cr.Status.LastSyncGeneration = origCR.Generation
+		if credentialsRootSecret != nil {
+			cr.Status.LastSyncCloudCredsSecretResourceVersion = credentialsRootSecret.ResourceVersion
+		}
 	}
 
 	err = r.updateStatus(origCR, cr, logger)
@@ -847,6 +801,8 @@ func crInfraMatches(cr *minterv1.CredentialsRequest, clusterCloudPlatform config
 		return cloudType == reflect.TypeOf(minterv1.OvirtProviderSpec{}).Name(), nil
 	case configv1.VSpherePlatformType:
 		return cloudType == reflect.TypeOf(minterv1.VSphereProviderSpec{}).Name(), nil
+	case configv1.KubevirtPlatformType:
+		return cloudType == reflect.TypeOf(minterv1.KubevirtProviderSpec{}).Name(), nil
 	default:
 		return false, fmt.Errorf("unsupported platorm type: %v", clusterCloudPlatform)
 	}
